package com.qmth.distributed.print.business.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.boot.tools.excel.ExcelReader;
import com.qmth.boot.tools.excel.enums.ExcelType;
import com.qmth.boot.tools.excel.model.DataMap;
import com.qmth.distributed.print.business.entity.TCUsualScore;
import com.qmth.distributed.print.business.mapper.TCUsualScoreMapper;
import com.qmth.distributed.print.business.service.PrintCommonService;
import com.qmth.distributed.print.business.service.TCUsualScoreService;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.FieldUniqueEnum;
import com.qmth.teachcloud.common.util.JacksonUtil;
import com.qmth.teachcloud.common.util.ResultUtil;
import com.qmth.teachcloud.common.util.ServletUtil;
import com.qmth.teachcloud.mark.entity.MarkPaper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.*;

/**
 * <p>
 * 平常作业成绩表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2024-02-18
 */
@Service
public class TCUsualScoreServiceImpl extends ServiceImpl<TCUsualScoreMapper, TCUsualScore> implements TCUsualScoreService {
    private final static Logger log = LoggerFactory.getLogger(TCUsualScoreServiceImpl.class);

    @Resource
    PrintCommonService printCommonService;

    @Resource
    TCUsualScoreService tcUsualScoreService;

    /**
     * 导入平时成绩excel
     *
     * @param file
     * @param examId
     * @param courseCode
     * @param paperNumber
     * @return
     * @throws IOException
     */
    @Override
    @Transactional
    public Map<String, String> usualScoreExcelImport(MultipartFile file, Long examId, String courseCode, String paperNumber) throws IOException {
        log.debug("导入Excel开始...");
        long start = System.currentTimeMillis();
        Map<String, String> messageMap = new LinkedHashMap<>();
        try {
            MarkPaper markPaper = printCommonService.scoreImportExcelVaild(file, examId, paperNumber);
            StringJoiner errorData = new StringJoiner("");
            StringJoiner successData = new StringJoiner("");

            ExcelReader excelReader = ExcelReader.create(ExcelType.XLSX, file.getInputStream(), 1);
            List<DataMap> list = excelReader.getDataMapList();
            log.info("list:{}", JacksonUtil.parseJson(list));

            SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
            List<TCUsualScore> tcUsualScoreList = new ArrayList<>(list.size());
            String[] columnNames = excelReader.getColumnNames();
            for (int i = 0; i < columnNames.length; i++) {
                if (i == 0 && Objects.nonNull(columnNames[i]) && !Objects.equals(columnNames[i].trim(), "学号")) {
                    throw ExceptionResultEnum.ERROR.exception("excel表头第一行为学号");
                } else if (i == 1 && Objects.nonNull(columnNames[i]) && !Objects.equals(columnNames[i].trim(), "姓名")) {
                    throw ExceptionResultEnum.ERROR.exception("excel表头第二行为姓名");
                }
            }

            for (int i = 0; i < list.size(); i++) {
                Map<String, String> objectMap = list.get(i);
                if (!CollectionUtils.isEmpty(objectMap)) {
                    boolean error = false;
                    JSONArray jsonArray = new JSONArray();
                    TCUsualScore tcUsualScore = new TCUsualScore(examId, courseCode, markPaper.getCourseName(), paperNumber, sysUser.getId());
                    for (Map.Entry<String, String> entry : objectMap.entrySet()) {
                        JSONObject jsonObject = new JSONObject();
                        if (Objects.isNull(entry.getValue()) || Objects.equals(entry.getValue().trim(), "")) {
                            errorData.add("excel第" + (i + 1) + "行[").add(entry.getKey() + "]为空;").add("\r\n");
                            error = true;
                        } else {
                            if (Objects.equals(entry.getKey().trim(), "学号")) {
                                tcUsualScore.setStudentCode(entry.getValue());
                            } else if (Objects.equals(entry.getKey().trim(), "姓名")) {
                                tcUsualScore.setName(entry.getValue());
                            } else {
                                jsonObject.put("name", entry.getKey());
                                jsonObject.put("score", entry.getValue());
                                jsonArray.add(jsonObject);
                            }
                        }
                    }
                    if (jsonArray.size() > 0) {
                        tcUsualScore.setScore(jsonArray.toJSONString());
                    }
                    if (!error) {
                        tcUsualScoreList.add(tcUsualScore);
                    }
                }
            }
            if (!CollectionUtils.isEmpty(tcUsualScoreList)) {
                successData.add("共导入" + tcUsualScoreList.size() + "条数据");
                tcUsualScoreService.remove(new QueryWrapper<TCUsualScore>().lambda().eq(TCUsualScore::getExamId, examId)
                        .eq(TCUsualScore::getCourseCode, courseCode)
                        .eq(TCUsualScore::getCourseName, markPaper.getCourseName())
                        .eq(TCUsualScore::getPaperNumber, paperNumber));
                tcUsualScoreService.saveBatch(tcUsualScoreList);
            }
            messageMap.put(SystemConstant.SUCCESS, successData.length() > 0 ? successData.toString() : "无");
            messageMap.put(SystemConstant.EXCEL_ERROR, errorData.length() > 0 ? errorData.toString() : "无");
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (e instanceof DuplicateKeyException) {
                String errorColumn = e.getCause().toString();
                String columnStr = errorColumn.substring(errorColumn.lastIndexOf("key") + 3, errorColumn.length()).replaceAll("'", "");
                throw ExceptionResultEnum.SQL_ERROR.exception("[" + FieldUniqueEnum.convertToTitle(columnStr) + "]数据不允许重复插入");
            } else if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, ((ApiException) e).getCode(), e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        log.debug("导入Excel结束...");
        long end = System.currentTimeMillis();
        log.info("============耗时{}秒============:", (end - start) / 1000);
        return messageMap;
    }

    /**
     * 平时成绩列表（分页）
     *
     * @param iPage
     * @param examId
     * @param courseCode
     * @param paperNumber
     * @return
     */
    @Override
    public IPage<TCUsualScore> usualScoreList(IPage<Map> iPage, Long examId, String courseCode, String paperNumber) {
        return this.baseMapper.usualScoreList(iPage, examId, courseCode, paperNumber);
    }
}
